# Example response from AWS for Aws::AutoScaling::Instance:
#
#   #<Aws::AutoScaling::Instance:0x00007fc8a2781438
#    @client=#<Aws::AutoScaling::Client>,
#    @data=
#     #<struct Aws::AutoScaling::Types::Instance
#      instance_id="i-007b9fa2f7df584f7",
#      instance_type="t3.medium",
#      availability_zone="us-west-2b",
#      lifecycle_state="InService",
#      health_status="Healthy",
#      launch_configuration_name=
#       "h1-testserver-lc-asg-rebooter-20191219124104839000000001",
#      launch_template=nil,
#      protected_from_scale_in=false,
#      weighted_capacity=nil>,
#    @group_name="h1-testserver-asg-asg-rebooter",
#    @id="i-007b9fa2f7df584f7">

module Asg
  module Rebooter
    module Wrappers
      class Instance
        LIFECYCLE_STANDBY = 'Standby'
        LIFECYCLE_IN_SERVICE = 'InService'
        LIFECYCLE_CHANGE_POLL_TIMEOUT = 4

        HEALTH_STATUS_HEALTHY = 'HEALTHY'

        SLEEP_AFTER_REBOOT = 30
        SLEEP_AFTER_STANDBY = 10

        POLL_TIMEOUT = 4

        attr_reader :instance

        def initialize(instance)
          #
          # @instance is of type Aws::AutoScaling::Instance
          #
          # #ec2_instance is of type Aws::EC2::Instance
          #
          @instance = instance
        end

        def method_missing(m, *args, &block)
          instance.send(m, *args, &block)
        end

        def enter_standby
          # raise when in invalid state for action

          instance.enter_standby(should_decrement_desired_capacity: true)

          ensure_lifecycle_state(LIFECYCLE_STANDBY)
        end

        def exit_standby
          # raise if in invalid state for action

          instance.exit_standby

          ensure_lifecycle_state(LIFECYCLE_IN_SERVICE)
        end

        def ensure_lifecycle_state(state)
          loop do
            instance.reload

            break if instance.lifecycle_state == state

            sleep LIFECYCLE_CHANGE_POLL_TIMEOUT
          end

          true
        end

        def standby?
          instance.reload

          instance.lifecycle_state == LIFECYCLE_STANDBY
        end

        def health_status
          instance.reload.health_status
        end

        def healthy?
          health_status == HEALTH_STATUS_HEALTHY
        end

        def wait_for_healthy
          #
          # TODO: Should check health state on the TargetGroup to ensure the instance is ready for work
          #
          loop do
            break if healthy?

            sleep(POLL_TIMEOUT)
          end

          true
        end

        def reboot
          ec2_instance.reboot
        end

        private

        def ec2_instance
          Aws::EC2::Instance.new(instance.instance_id)
        end
      end
    end
  end
end
